import logging
import geopandas as gpd

def valued_dpm_shp_postpro(valued_shp_file,average_value):
    # 2025-2-1 07:53:52 long.liu89@hotmail.com
    # 依据赋值后的shp计算变量处方的总施肥量；
    # 1. 计算shp中每个polygon的面积以及施肥量，对其进行加和，得到变量施肥的总施量；
    # 2. 计算shp中每个polygon的面积，进行加和,再乘以单位面积上的施肥量，该值由 self.average_fertilization_value 来计算，得到匀量施肥的总施肥量；
    # 3. 基于变量施肥的总施肥量以及匀量施肥的总施肥量，得到节肥率；
    # 计算变量施肥总施量和总面积

    """直接根据属性表中的 value 字段计算施肥总量和节肥率"""
    try:
        # 1. 读取处方图 Shapefile
        gdf = gpd.read_file(valued_shp_file)

        # 2. 校验必要字段存在
        required_fields = ["value", "area"]
        for field in required_fields:
            if field not in gdf.columns:
                raise KeyError(f"Shapefile 缺少必要字段: {field}")

        # 3. 计算变量施肥总施量
        variable_total = 0.0
        total_area = 0.0

        for _, row in gdf.iterrows():
            value = row["value"]  # 单位面积施肥量（kg/ha）
            area = row["area"]  # 面积（公顷）

            # 数值有效性检查
            if not (isinstance(value, (int, float)) and isinstance(area, (int, float))):
                raise ValueError("字段 value 或 area 包含非数值类型")

            variable_total += value * area
            total_area += area

        # 4. 计算匀量施肥总施量（需确保类中已定义 self.average_fertilization_value）
        uniform_total = total_area * average_value

        # 5. 计算节肥率（避免除零错误）
        saving_rate = 0.0
        if uniform_total > 0:
            saving_rate = (1 - variable_total / uniform_total) * 100

        # 保存结果并保留两位小数
        variable_total = round(variable_total, 2)
        uniform_total = round(uniform_total, 2)
        saving_rate = round(saving_rate, 2)

        # 输出日志
        logging.info(
            f"变量施肥总量: {variable_total} kg\n"
            f"匀量施肥总量: {uniform_total} kg\n"
            f"节肥率: {saving_rate}%"
        )
        return variable_total,uniform_total,saving_rate
    except Exception as e:
        logging.error(f"计算施肥量失败: {str(e)}")
        raise



import datetime
import os


class NdviFiles(object):
    def __init__(
        self,
        work_dirpath: str = None,
    ):
        """初始化 解析ndvi_tif 文件

        Args:
            ndvi_tif (str): ndvi tif 文件路径
            values: 分级数据值
            boundary_geometry: 边界 geometry,与边界 shp文件 二选一
            boundary_shp_filepath: 边界shp文件,与边界geometry 二选一
            work_dirpath: 工作路径,生成的过程文件在此目录
        """
        if work_dirpath is None:
            work_dirpath = "./data"
        if not os.path.exists(work_dirpath):
            os.makedirs(work_dirpath, exist_ok=True)
        self._timesuffix = datetime.datetime.now().strftime("%Y%m%d%H%M%S")

        self._reclassify_file = os.path.join(
            work_dirpath, f"reclassify-{self._timesuffix}.tif"
        )

        self._dpm_shp_file = os.path.join(
            work_dirpath, f"dpm_shp_file-{self._timesuffix}.shp"
        )

        self._valued_shp_file = os.path.join(
            work_dirpath, f"valued_dpm_shp_file-{self._timesuffix}.shp"
        )

        self._geojson_file = os.path.join(
            work_dirpath, f"prescription-{self._timesuffix}.geojson"
        )

        self._clipped_shp_file = os.path.join(
            work_dirpath, f"clipped_dpm_shp_file-{self._timesuffix}.shp"
        )

        pass






import logging
import math

import geopandas as gpd


import rasterio.sample  # 否则pyinstaller打包时会报错;
import rasterio.vrt  # 否则pyinstaller打包时会报错;
import rasterio._features  # 否则pyinstaller打包时会报错;

from rasterstats import zonal_stats

def get_zone_indexes( geotiff_file, shp_file):
    """
    Calculate the average NDVI or other pixel values for each unique value in the shapefile,
    based on the GeoTIFF raster data.

    :param geotiff_file: Path to the GeoTIFF file (e.g., NDVI data).
    :param shp_file: Path to the shapefile that defines zones with 'value' attribute.
    :return: A dictionary where keys are unique 'value' attributes and values are the average NDVI for each.
    """
    try:
        # 读取形状文件 (Shapefile)
        zones_gdf = gpd.read_file(shp_file)

        # 确保 'value' 字段存在
        if "value" not in zones_gdf.columns:
            raise ValueError("'value' column not found in the Shapefile")

        # 读取 GeoTIFF 文件
        with rasterio.open(geotiff_file) as src:
            affine = src.transform  # 获取地理变换信息
            raster = src.read(1)  # 读取第一波段数据，假设 NDVI 或其他指数在第一波段

        # 获取唯一的 value 属性
        unique_values = zones_gdf["value"].unique()

        # 存储每个 'value' 的平均值
        value_mean_dict = {}

        # 对每个 'value' 进行 zonal_stats 计算
        for val in unique_values:
            # 提取 Shapefile 中与当前 'value' 对应的多边形
            value_zones = zones_gdf[zones_gdf["value"] == val]

            # 计算该 'value' 区域的栅格平均值
            zone_stats = zonal_stats(
                value_zones,
                raster,
                affine=affine,
                stats=["mean"],
                nodata=src.nodata,
            )

            # 提取统计结果的平均值
            value_mean = sum(
                [stat["mean"] for stat in zone_stats if stat["mean"] is not None]
            ) / len(zone_stats)

            # 将平均值存入字典
            value_mean_dict[val] = value_mean

        # 输出每个 'value' 的平均值（NDVI 等）
        for value, mean_value in value_mean_dict.items():
            logging.info(f"Value {value} average NDVI: {mean_value}")

        return value_mean_dict

    except Exception as e:
        raise Exception(f"Error calculating zone indexes: {e}")





# 创建class Spraying
# long.liu89@hotmail.com
# 2025-2-2 10:47:34
import logging
import math
import rasterio
import geopandas as gpd
from rasterstats import zonal_stats
from .common import get_zone_indexes
class Spraying:
    # 作物类型与耐药性系数（假设值，需根据实际数据调整）
    crop_resistance = {
        "wheat": 0.8,    # 小麦耐药性较高
        "maize": 0.6,    # 玉米耐药性中等
    }
    # 农药类型与推荐剂量（单位：g/ha，示例数据）
    pesticide_dose = {
        "insecticide": {"min": 200, "max": 400},  # 杀虫剂
        "fungicide": {"min": 150, "max": 300},    # 杀菌剂
        "herbicide": {"min": 100, "max": 250},    # 除草剂
    }
    # 病虫害风险等级与剂量调整系数（假设逻辑）
    risk_adjustment = {
        "low": 0.5,     # 低风险减少剂量
        "medium": 1.0,  # 中风险标准剂量
        "high": 1.5,    # 高风险增加剂量
    }

    def __init__(
        self,
        # --- 农药相关参数 ---
        pesticide_type="fungicide",  # 农药类型
        pesticide_concentration=50,    # 有效成分浓度（%）
        # recommended_dose=300,          # 推荐剂量（g/ha）
        recommended_dose = 2,       #推荐剂量 (L/mu),mu: 亩
        pest_risk_level="medium",      # 病虫害风险等级
        
        # --- 作物相关参数 ---
        crop_type="maize",
        crop_phenology="tasseling_maize",  # 作物生育期，作物生育期影响着抗性
        
        # --- 环境参数 ---
        weather_precipitation=0.0,     # 未来24小时降雨量（mm）
        weather_temperature=25.0,      # 当前温度（℃）
        
        # --- 地理数据参数 ---
        # geotiff_file=None,             # 遥感数据文件（如病虫害风险图）
        # shp_file=None,                 # 分区形状文件
    ) -> None:
        
        # --- 农药参数初始化 ---
        self.pesticide_type = pesticide_type
        self.pesticide_concentration = pesticide_concentration
        self.recommended_dose = recommended_dose
        self.pest_risk_level = pest_risk_level
        
        # --- 作物参数初始化 ---
        self.crop_type = crop_type
        self.crop_phenology = crop_phenology
        self.crop_resistance = self.crop_resistance.get(crop_type, 1.0)
        
        # --- 环境参数初始化 ---
        self.weather_precipitation = weather_precipitation
        self.weather_temperature = weather_temperature
        
        # # --- 地理数据处理 ---
        # self.geotiff_file = geotiff_file
        # self.shp_file = shp_file

    # 目前传过来的是NDVI，其实后面要改成用 raw_geotiff_file 来搞；
    def risk_based_pesticide_spraying(self, ndvi_geotiff_file, valued_shp_file, recommended_dose):

        # 读入 ndvi_geotiff_file

        # 将 zone_shp_file 和 ndvi_geotiff_file 进行匹配，得到每个 zone 的 NDVI 值
        
        # 将 NDVI 转化为 risk level

        # 根据 risk level 以及天气条件计算实际农药用量

        zone_indexes = get_zone_indexes(ndvi_geotiff_file,valued_shp_file)
        #
        pesticide_dose_values = []
        # for index, zone_ndvi in enumerate(self.zone_indexes):
        for (
            index,
            zone_ndvi,
        ) in zone_indexes.items():  # 使用 .items() 来获取键和值
            zone_pesticide_dose = recommended_dose * (
                1 - zone_ndvi
            )  
            # 根据 NDVI 调整施药量
            # if zone_ndvi < 0.3:
            #     zone_pesticide_dose *= 1.5
            # elif zone_ndvi >= 0.3 and zone_ndvi < 0.6:
            #     zone_pesticide_dose *= 1.0
            # elif zone_ndvi >= 0.6:
            #     zone_pesticide_dose *= 0.5

            zone_pesticide_dose = float(math.ceil(zone_pesticide_dose*100.0)/100.0)
            pesticide_dose_values.append(zone_pesticide_dose)
            logging.info(
                f"Pesticide required for zone {index + 1} (NDVI={zone_ndvi}): {zone_pesticide_dose:.2f} L/mu"
            )

        pesticide_dose_values.sort()
        return pesticide_dose_values, recommended_dose
    
    def calculate_pesticide_amount(self):
        """
        根据病虫害风险等级、作物耐药性和天气条件计算实际农药用量。
        返回：
        农药用量（g/ha）
        """
        # 基础剂量调整
        base_dose = self.recommended_dose * self.risk_adjustment.get(self.pest_risk_level, 1.0)
        
        # 根据作物耐药性调整
        adjusted_dose = base_dose / self.crop_resistance
        
        # 根据天气调整（降雨可能降低药效）
        if self.weather_precipitation > 10:
            adjusted_dose *= 1.2  # 降雨量大则增加10%剂量
        elif self.weather_temperature > 30:
            adjusted_dose *= 0.9  # 高温减少剂量
        
        # 确保在农药类型的安全范围内
        dose_range = self.pesticide_dose.get(self.pesticide_type, {"min": 0, "max": 0})
        final_dose = max(dose_range["min"], min(adjusted_dose, dose_range["max"]))
        
        return round(final_dose, 2)

    def generate_spraying_map(self):
        """
        根据分区地理数据生成差异化喷洒处方图。
        返回：
        各区域的农药用量字典 {zone_id: dose_g_per_ha}
        """
        if not self.geotiff_file or not self.shp_file:
            raise ValueError("GeoTIFF或SHP文件未提供")
        
        zone_risk = self.get_zone_risk_indexes(self.geotiff_file, self.shp_file)
        spraying_map = {}
        
        for zone_id, risk_value in zone_risk.items():
            # 假设risk_value是0-1的病虫害风险值
            if risk_value < 0.3:
                self.pest_risk_level = "low"
            elif risk_value < 0.7:
                self.pest_risk_level = "medium"
            else:
                self.pest_risk_level = "high"
            
            dose = self.calculate_pesticide_amount()
            spraying_map[zone_id] = dose
        
        return spraying_map

    def get_zone_risk_indexes(self, geotiff_file, shp_file):
        """
        从GeoTIFF中提取各分区的病虫害风险指数。
        """
        try:
            zones_gdf = gpd.read_file(shp_file)
            if "value" not in zones_gdf.columns:
                raise ValueError("Shapefile缺少'value'字段")
            
            with rasterio.open(geotiff_file) as src:
                raster = src.read(1)
                affine = src.transform
            
            zone_stats = zonal_stats(
                zones_gdf,
                raster,
                affine=affine,
                stats=["mean"],
                nodata=src.nodata
            )
            
            risk_indexes = {
                row["value"]: stats["mean"] if stats["mean"] else 0
                for row, stats in zip(zones_gdf.to_dict("records"), zone_stats)
            }
            
            return risk_indexes
        
        except Exception as e:
            raise Exception(f"风险指数计算失败: {e}")



import logging
import geopandas as gpd

from .files import NdviFiles
from ...utils.geotiff_toolkit.reclassify import reclassify, raster2vector
from ...utils.shp_toolkit.toolkit import clip_shp_by_polygon, ShpToolkit
from ..generatevalues.irrigation import water_balance_calculation_base_irrigation
from .common import valued_dpm_shp_postpro


class IrrigationPrescription:
    def __init__(
        self,
        ndvi_tif,
        geometry: dict,
        num=5,
        workspace=".",
        precipitation=50,
        evapotranspiration=60,
        runoff=10,
        deep_percolation=5,
        initial_soil_moisture=280,
        target_soil_moisture=300,
    ):
        """
        ndvi_tif: ndvi geotif 文件
        precipitation = 50,  # mm precipitation (float): 降水量 (毫米)
        evapotranspiration = 60,  # mm 蒸发蒸腾量 (毫米)
        runoff = 10,  # mm 地表径流量 (毫米)
        deep_percolation = 5,  # mm 深层渗漏量 (毫米)
        initial_soil_moisture = 280,  # mm 初始土壤含水量 (毫米)
        target_soil_moisture = 300, #mm 目标土壤含水量(毫米)
        """

        self._ndvi_tif = ndvi_tif
        self._num = num
        self._geometry = geometry
        self._workspace = workspace
        self._files = NdviFiles(self._workspace)
        # 灌溉相关
        self.precipitation = precipitation  # mm precipitation (float): 降水量 (毫米)
        self.evapotranspiration = evapotranspiration  # mm 蒸发蒸腾量 (毫米)
        self.runoff = runoff  # mm 地表径流量 (毫米)
        self.deep_percolation = deep_percolation  # mm 深层渗漏量 (毫米)
        self.initial_soil_moisture = initial_soil_moisture  # mm 初始土壤含水量 (毫米)
        self.target_soil_moisture = target_soil_moisture  # 目标土壤含水量 (毫米)

    def run(self):
        # reclassify
        reclassify(self._ndvi_tif, self._files._reclassify_file, level_num=self._num)
        raster2vector(self._files._reclassify_file, self._files._dpm_shp_file)

        # 首先需要根据处方图的类型，判断调用哪种处方图的计算函数，是底肥，追肥；
        # 追肥的话，这里需要添加养分平衡方程，而且需要先对接口传过来的数据进行判断，
        # 如果接口传过来的数据中values值不是空，那么就调用这个函数，
        # 否则就调用养分平衡方程进行计算，需要知道土壤里面有多少氮、磷、钾（碱解氮、速效磷、速效钾需要进行换算），
        # 目前这种 crop_type （作物类型）处在这种生育期下应该需要多少氮、磷、钾，结合此种作物当前生育期下的吸收率；
        # 还需要考虑上次施肥剩下的氮，是速效肥还是缓释肥，考虑气象条件，目前还剩下多少氮、磷、钾。最后计算出来应该追多少肥。底肥的话，只考虑土壤及作物品种就行。

        # 氮的需求量应该还受NDVI的调控；
        # self.agri_operation_input_values = self.nutrients_balance_calculation_base_fertilization()

        # 这里选择变量灌溉量的计算方法：人工分段指定法；节水率法；水量平衡法；
        # 应该还有旱灾等级法；
        values, average_value = water_balance_calculation_base_irrigation(
            self._ndvi_tif,
            self._files._dpm_shp_file,
            self.precipitation,
            self.evapotranspiration,
            self.runoff,
            self.deep_percolation,
            self.initial_soil_moisture,
            self.target_soil_moisture,
        )
        clip_shp_by_polygon(
            self._files._dpm_shp_file, self._geometry, self._files._clipped_shp_file
        )
        shtool = ShpToolkit(self._files._clipped_shp_file)
        shtool.insert_attributes(values=values)
        shtool.save(self._files._valued_shp_file)
        shtool.save(self._files._geojson_file, driver="GeoJSON")

        # print(f"推荐灌溉量为 {recommended_irrigation} 升水每亩")

        variable_total, uniform_total, saving_rate = valued_dpm_shp_postpro(
            self._files._valued_shp_file, average_value
        )  # 依据生成的处方图，计算处方作业总施肥量、变量作业总施肥量、节肥率；

        logging.info(
            f"变量施肥总量: {variable_total} kg\n"
            f"匀量施肥总量: {uniform_total} kg\n"
            f"节肥率: {saving_rate}%"
        )



from fiona.crs import from_epsg
import geopandas as gpd

from shapely.geometry import Polygon, MultiPolygon, GeometryCollection

class ShpToolkit():
    def __init__(self, shp_filepath):

        self._gdf = gpd.read_file(shp_filepath)
        pass

    def insert_attributes(
        self,
        colors: list = [
            "#D2DE6A",
            "#F1F143",
            "#C3E910",
            "#53E41D",
            "#1FB50D",
            "#227329",
            "#165B1B",
            "#011006",
        ],
        values: list=None,
        sorted_by: str ="value",
        desc: bool = True,
    ):
        """
        给定一个shp文件，给定三个值，将shp文件中的属性值进行分类
        参数:
        input_dpm_shp_file (str): 输入的shp文件名
        output_valued_dpm_shp_file (str): 输出的赋值后的dpm的shp文件名
        """

        # 获取value列中从大到小的五个不重复数
        sorted_values = self._gdf[sorted_by].unique()
        sorted_values.sort()  # 按值排序
    
        # 处方图 倒序
        if desc:
            sorted_values = sorted_values[::-1]  # 从大到小排序

        def level_mapper(val):
            return sorted_values.tolist().index(val) + 1

        def color_mapper(val):
            index = sorted_values.tolist().index(val)
            if index >= len(colors):
                return colors[-1]
            return colors[index]

        def value_mapper(val):
            return values[sorted_values.tolist().index(val)]

        # 将颜色信息添加到GeoDataFrame中
        self._gdf["color"] = self._gdf[sorted_by].apply(color_mapper)
        self._gdf["level"] = self._gdf[sorted_by].apply(level_mapper)

        # value 转换为指定的施肥量
        if values :
            self._gdf["value"] = self._gdf[sorted_by].apply(value_mapper)
        # gdf["color"] = np.select(conditions, choice_colors, default=colors[0])
        # Check if CRS is missing (None)
        # long.liu89@hotmail.com 2024-5-9 14:24:08
        epsg = 4326
        if self._gdf.crs is None:
            # Assign a default CRS (e.g., EPSG:4326)
            self._gdf.crs = from_epsg(epsg)  # Assign EPSG code to the CRS

            # # Generate new filename based on the original shapefile path
            # path, filename = os.path.split()
            # filename_no_ext, ext = os.path.splitext(filename)
            # new_filename = os.path.join(path, f"{filename_no_ext}_{epsg}{ext}")

            # Save the GeoDataFrame to the new shapefile path

            # 保存分类后的shp文件
        self._gdf["area"] = round(self._gdf.to_crs(32649).area / 10000, 6)
        self._gdf["area_total"] = self._gdf["area"]
        self._gdf["area_unit"] = "ha"
        # grouped_areas = round(gdf.groupby('level').area.sum(),6)
        # # for geometry_type, area in grouped_areas.items():
        # #     print(f"level: {geometry_type}, Total Area (m²): {area}")
        # for geometry_type, area in grouped_areas.items():
        #     gdf.loc[gdf['level'] == geometry_type, 'area_total'] = area

        self._gdf = self._gdf.dissolve(
            by="level",
            aggfunc={
                "color": "first",
                "value": "first",
                "area": "sum",
                "area_unit": "first",
                "area_total": "sum",
            },
        )

        # gdf.to_file(self._valued_shp_file)
        # gdf.to_file(self._geojson_file, driver="GeoJSON")

    def save(self, filename,driver=None):
        self._gdf.to_file(filename=filename,driver=driver)


def clip_shp_by_polygon(input_shp, clip_polygon_json, output_shp):
    # 读取待裁剪的shp文件
    to_be_clipped = gpd.read_file(input_shp)

    # 定义裁剪用的多边形
    polygon_coords = clip_polygon_json["coordinates"][0]
    clipping_polygon = Polygon(polygon_coords)

    # 确保多边形和shp文件在同一坐标参考系统（CRS）
    # 如果CRS不同，需要进行转换
    if to_be_clipped.crs != "EPSG:4326":
        clipping_polygon = gpd.GeoSeries([clipping_polygon], crs="EPSG:4326")
        clipping_polygon = clipping_polygon.to_crs(to_be_clipped.crs).iloc[0]

    # 裁剪操作
    clipped = to_be_clipped.clip(clipping_polygon)

    # 处理不同的几何类型
    valid_geometries = []
    valid_records = []

    for idx, geom in enumerate(clipped.geometry):
        if geom.geom_type == "Polygon":
            valid_geometries.append(geom)
            valid_records.append(clipped.iloc[idx])
        elif geom.geom_type == "MultiPolygon":
            # 使用 .geoms 属性来遍历 MultiPolygon 中的每个 Polygon
            for poly in geom.geoms:
                valid_geometries.append(poly)
                valid_records.append(clipped.iloc[idx])
        elif geom.geom_type == "GeometryCollection":
            # 从 GeometryCollection 中提取多边形
            for sub_geom in geom.geoms:
                if sub_geom.geom_type == "Polygon":
                    valid_geometries.append(sub_geom)
                    valid_records.append(clipped.iloc[idx])

    # 使用有效的几何和属性构建新的 GeoDataFrame
    valid_df = gpd.GeoDataFrame(valid_records, geometry=valid_geometries, crs=to_be_clipped.crs)

    # 检查是否为空，避免保存空文件
    if valid_df.empty:
        print("The clipped result is empty. No output file will be created.")
    else:
        # 保存裁剪后的shp文件
        valid_df.to_file(output_shp)
    
def clip_shp_by_shp(input_shp, clip_shp, output_shp):
    """
    使用一个 shp 文件裁剪另一个 shp 文件

    参数:
    input_shp (str): 输入的 shp 文件路径
    clip_shp (str): 用来裁剪的 shp 文件路径
    output_shp (str): 输出的裁剪后的 shp 文件路径
    """
    # 读取输入的 shp 文件和用来裁剪的 shp 文件
    input_gdf = gpd.read_file(input_shp)

    clip_gdf = gpd.read_file(clip_shp)

    # 进行空间裁剪
    clipped_gdf = gpd.overlay(input_gdf, clip_gdf, how="intersection")

    # 将裁剪后的结果保存为新的 shp 文件
    clipped_gdf.to_file(output_shp)
    


import logging

import math
from .common import get_zone_indexes


class Fertilization:
    # 根据作物类型设置每千克产量的氮磷钾需求量
    crop_demand_g_per_kg = {
        "wheat": {"nitrogen": 25.0, "porphorus": 12.0, "kassium": 20.0},
        "maize": {"nitrogen": 25.0, "porphorus": 12.0, "kassium": 20.0},
    }
    # 根据作物类型和生育期设置氮需求量比例
    nitrogen_demand_ratio = {
        "wheat": {
            "seeding_wheat": 0.15,  # 苗期
            "tillering_wheat": 0.35,  # 分蘖期
            "jointing_wheat": 0.25,  # 拔节期
            "booting_wheat": 0.15,  # 抽穗期
            "grain_filling_wheat": 0.10,  # 灌浆期
        },
        "maize": {
            "emergence_maize": 0.10,  # 出苗期
            "vegetative_maize": 0.40,  # 营养生长期
            "tasseling_maize": 0.30,  # 抽雄期
            "grain_filling_maize": 0.20,  # 灌浆期
        },
    }
    # 根据作物类型和生育期设置钾需求量比例
    kassium_demand_ratio = {
        "wheat": {
            "seeding_wheat": 0.15,  # 苗期
            "tillering_wheat": 0.35,  # 分蘖期
            "jointing_wheat": 0.25,  # 拔节期
            "booting_wheat": 0.15,  # 抽穗期
            "grain_filling_wheat": 0.10,  # 灌浆期
        },
        "maize": {
            "emergence_maize": 0.10,  # 出苗期
            "vegetative_maize": 0.40,  # 营养生长期
            "tasseling_maize": 0.30,  # 抽雄期
            "grain_filling_maize": 0.20,  # 灌浆期
        },
    }
    # 根据作物类型和生育期设置磷需求量比例
    porphorus_demand_ratio = {
        "wheat": {
            "seeding_wheat": 0.15,  # 苗期
            "tillering_wheat": 0.35,  # 分蘖期
            "jointing_wheat": 0.25,  # 拔节期
            "booting_wheat": 0.15,  # 抽穗期
            "grain_filling_wheat": 0.10,  # 灌浆期
        },
        "maize": {
            "emergence_maize": 0.10,  # 出苗期
            "vegetative_maize": 0.40,  # 营养生长期
            "tasseling_maize": 0.30,  # 抽雄期
            "grain_filling_maize": 0.20,  # 灌浆期
        },
    }

    def __init__(
        self,
        fertilizer_type="base",  # 基肥
        fertilizer_form="mixed",  # 混合肥 or 单质肥
        fertilizer_npk_ratio=[15, 15, 15],  # NPK 比例
        fertilizer_average_value_manual=375,  # kg/ha 人工输入的平均施肥量
        # --- crop related variables ---
        crop_type="maize",
        crop_phenology="tasseling_maize",  #'tillering',
        # 未用
        crop_variaties="明科玉77",  #'鲁单9088', 未用
        crop_target_yield_kg_per_ha=10000,  # kg/ha, ex: 10000 kg/ha = 10 t/ha
        crop_nitrogen_use_efficiency=0.3,  # 土壤有关?
        # --- crop porphorus related variables ---
        crop_porphorus_use_efficiency=0.3,
        # --- crop kassium related variables ---
        crop_kassium_use_efficiency=0.3,
        # 未用 追肥
        # --- weather related variables ---
        weather_station_id="CN101010100",
        weather_station_latitude=48.908995,
        weather_station_longitude=125.207581,
        weather_station_elevation=0.0,
        # weather_station_timezone = 'Asia/Shanghai',
        # weather_station_timezone_offset = 8.0,
        # weather_station_timezone_dst = 0.0,
        # weather_station_timezone_dst_start = '2023-03-08',
        # weather_station_timezone_dst_end = '2023-11-01',
        # weather_station_timezone_dst_start_offset = 0.0,
        # weather_station_timezone_dst_end_offset = 0.0,
        # weather_cumulative_rainfall = 0.0,
        # weather_cumulative_snowfall = 0.0,
        weather_cumulative_temperature=0.0,
        # weather_cumulative_windspeed = 0.0,
        # weather_cumulative_humidity = 0.0,
        weather_cumulative_precipitation=0.0,
        # weather_cumulative_snowdepth = 0.0,
        # weather_cumulative_snowmelt = 0.0,
        # --- soil related variables ---
        soil_alkali_hydrolyzable_n_mg_per_kg=191.32,  # mg/kg
        soil_available_p_mg_per_kg=34.251,  # mg/kg
        soil_available_k_mg_per_kg=250.177,  # mg/kg
        soil_bulk_density_kg_per_m3=1300,  # kg/m^3
        soil_layer_depth_cm=20,  # cm; 只考虑20cm耕层的土壤养分
        # 未用  基肥
        soil_type="sandy loam",
        soil_pH=6.0,
        soil_sampling_latitude=48.908995,
        soil_sampling_longitude=125.207581,
        soil_sampling_date="2023-05-01T08:00:00Z",
        # soil_sampling_time="08:00:00",
        soil_sampling_method="sampling",
        soil_sampling_depth_cm=0.0,  # cm
        soil_sampling_temperature=0.0,
        soil_sampling_temperature_depth=0.0,
        soil_sampling_humidity=0.0,
        soil_sampling_humidity_depth=0.0,
        # --- fertilizer related variables ---
        fertilizer_npk_type="Nitrogen",
        fertilizer_amount=0.0,
        fertilizer_application_date="2023-05-01T08:00:00Z",
        # fertilizer_application_time="08:00:00",
        fertilizer_application_method="spraying",
        fertilizer_application_machine="T60",
    ) -> None:

        # --- dpm related variables ---

        self.fertilizer_type = fertilizer_type  # base;follow
        self.fertilizer_form = fertilizer_form  # single;mixed
        self.fertilizer_npk_ratio = fertilizer_npk_ratio  # for mixed
        self.fertilizer_average_value_manual = fertilizer_average_value_manual
        # --- crop related variables ---
        self.crop_type = crop_type
        self.crop_phenology = crop_phenology  #'tillering',
        self.crop_variaties = crop_variaties  #'鲁单9088',
        self.crop_target_yield_kg_per_ha = (
            crop_target_yield_kg_per_ha  # kg/ha, ex: 10000 kg/ha = 10 t/ha
        )

        # --- crop nitrogen related variables ---
        self.crop_nitrogen_demand_g_per_kg = self.crop_demand_g_per_kg[crop_type][
            "nitrogen"
        ]  # pure N, 纯氮
        self.crop_nitrogen_use_efficiency = crop_nitrogen_use_efficiency

        # --- crop porphorus related variables ---
        self.crop_porphorus_demand_g_per_kg = self.crop_demand_g_per_kg[crop_type][
            "porphorus"
        ]
        self.crop_porphorus_use_efficiency = crop_porphorus_use_efficiency

        # --- crop kassium related variables ---
        self.crop_kassium_demand_g_per_kg = self.crop_demand_g_per_kg[crop_type][
            "kassium"
        ]  # K2O
        self.crop_kassium_use_efficiency = crop_kassium_use_efficiency

        # 从字典中获取当前作物类型及当前生育期对应的氮肥比例
        self.ratio_nitrogen = self.nitrogen_demand_ratio[crop_type][crop_phenology]
        # 从字典中获取当前作物类型及当前生育期对应的磷肥比例
        self.ratio_porphorus = self.porphorus_demand_ratio[crop_type][crop_phenology]
        # 从字典中获取当前作物类型及当前生育期对应的钾肥比例
        self.ratio_kassium = self.kassium_demand_ratio[crop_type][crop_phenology]

        # --- weather related variables ---
        self.weather_station_id = weather_station_id
        self.weather_station_latitude = weather_station_latitude
        self.weather_station_longitude = weather_station_longitude
        self.weather_station_elevation = weather_station_elevation
        # self.weather_station_timezone = 'Asia/Shanghai'
        # self.weather_station_timezone_offset = 8.0
        # self.weather_station_timezone_dst = 0.0
        # self.weather_station_timezone_dst_start = '2023-03-08'
        # self.weather_station_timezone_dst_end = '2023-11-01'
        # self.weather_station_timezone_dst_start_offset = 0.0
        # self.weather_station_timezone_dst_end_offset = 0.0
        # self.weather_cumulative_rainfall = 0.0
        # self.weather_cumulative_snowfall = 0.0
        self.weather_cumulative_temperature = weather_cumulative_temperature
        # self.weather_cumulative_windspeed = 0.0
        # self.weather_cumulative_humidity = 0.0
        self.weather_cumulative_precipitation = weather_cumulative_precipitation
        # self.weather_cumulative_snowdepth = 0.0
        # self.weather_cumulative_snowmelt = 0.0

        # --- soil related variables ---
        self.soil_type = soil_type
        self.soil_pH = soil_pH
        self.soil_alkali_hydrolyzable_n_mg_per_kg = (
            soil_alkali_hydrolyzable_n_mg_per_kg  # mg/kg
        )
        self.soil_available_p_mg_per_kg = soil_available_p_mg_per_kg  # mg/kg
        self.soil_available_k_mg_per_kg = soil_available_k_mg_per_kg  # mg/kg
        self.soil_bulk_density_kg_per_m3 = soil_bulk_density_kg_per_m3  # kg/m^3
        self.soil_layer_depth_cm = soil_layer_depth_cm  # cm; 只考虑20cm耕层的土壤养分
        self.soil_sampling_latitude = soil_sampling_latitude
        self.soil_sampling_longitude = soil_sampling_longitude
        self.soil_sampling_date = soil_sampling_date

        self.soil_sampling_method = soil_sampling_method
        self.soil_sampling_depth_cm = soil_sampling_depth_cm  # cm
        self.soil_sampling_temperature = soil_sampling_temperature
        self.soil_sampling_temperature_depth = soil_sampling_temperature_depth
        self.soil_sampling_humidity = soil_sampling_humidity
        self.soil_sampling_humidity_depth = soil_sampling_humidity_depth

        # --- fertilizer related variables ---
        self.fertilizer_npk_type = fertilizer_npk_type
        self.fertilizer_amount = fertilizer_amount
        self.fertilizer_application_date = fertilizer_application_date
        self.fertilizer_application_method = fertilizer_application_method
        self.fertilizer_application_machine = fertilizer_application_machine

    def calculate_fertilizer_amount(
        self, required_n, required_p, required_k, fertilizer_npk_ratio
    ):
        """
        根据氮、磷、钾的需求量计算混合肥的用量。

        参数:
        required_n: 需求氮量 (kg/ha)
        required_p: 需求磷量 (kg/ha)
        required_k: 需求钾量 (kg/ha)
        fertilizer_npk_ratio: 混合肥中 N-P-K 的比例 (例如 [15, 15, 15])

        返回:
        需要施用的混合肥量 (kg/ha)
        """
        n_ratio, p_ratio, k_ratio = fertilizer_npk_ratio

        # 分别计算满足每种养分需求所需的肥料量
        fertilizer_amount_n = required_n / (n_ratio / 100.0)
        fertilizer_amount_p = required_p / (p_ratio / 100.0)
        fertilizer_amount_k = required_k / (k_ratio / 100.0)

        # 返回满足所有养分需求所需的最大肥料量
        return max(fertilizer_amount_n, fertilizer_amount_p, fertilizer_amount_k)

    # 依据养分平衡方程，判断基肥的施肥量
    def nutrients_balance_calculation_base_fertilization(
        self, geotiff_file, valued_shp_file
    ):
        """
        Calculate the fertilizer requirements based on soil nutrient content, crop requirements, and residual nutrients.
        This function should be updated according to the specific crop type and growth stage.
        """

        # Example inputs: these would be fetched or calculated based on the crop and soil data
        # crop_type = "maize"  # Example crop type, could be passed as a parameter or part of the class
        # crop_phenology = "tillering"  # Example growth stage

        zone_indexes = get_zone_indexes(geotiff_file, valued_shp_file)

        crop_type = self.crop_type
        crop_phenology = self.crop_phenology

        # 计算总氮需求量
        crop_total_nitrogen_demand = (
            self.crop_target_yield_kg_per_ha * self.crop_nitrogen_demand_g_per_kg / 1000
        )  # kg 纯氮, 这几行代码上面应该添加针对氮需求量的描述；
        crop_total_porphorus_demand = (
            self.crop_target_yield_kg_per_ha
            * self.crop_porphorus_demand_g_per_kg
            / 1000
        )  # kg P2O5
        crop_total_kassium_demand = (
            self.crop_target_yield_kg_per_ha * self.crop_kassium_demand_g_per_kg / 1000
        )  # kg K2O

        # Soil nutrient levels (could be fetched from a database or input as parameters)
        # soil_n = 50  # Available Nitrogen (kg/ha)
        # soil_p = 20  # Available Phosphorus (kg/ha)
        # soil_k = 30  # Available Potassium (kg/ha)

        soil_n = (
            self.soil_alkali_hydrolyzable_n_mg_per_kg
            / 1000.0
            / 1000.0
            * 10000
            * self.soil_layer_depth_cm
            / 100.0
            / 2
            * self.soil_bulk_density_kg_per_m3
            * self.crop_nitrogen_use_efficiency
        )  # convert mg/kg to kg/ha
        # mg/kg            => k/kg => kg/kg => kg/kg*ha => kg/kg*10000m3 => kg/kg*10000m3 *kg/m3 => kg/ha
        soil_p = (
            self.soil_available_p_mg_per_kg
            / 1000.0
            / 1000.0
            * 10000
            * self.soil_layer_depth_cm
            / 100.0
            / 2
            * self.soil_bulk_density_kg_per_m3
            * self.crop_porphorus_use_efficiency
        )
        soil_k = (
            self.soil_available_k_mg_per_kg
            / 1000.0
            / 1000.0
            * 10000
            * self.soil_layer_depth_cm
            / 100.0
            / 2
            * self.soil_bulk_density_kg_per_m3
            * self.crop_kassium_use_efficiency
        )

        # # Crop nutrient requirements per growth stage (hypothetical values)
        # nutrient_requirements = {
        #     "wheat": {
        #         "tillering": {"N": 60, "P": 30, "K": 40},
        #         "flowering": {"N": 80, "P": 40, "K": 60},
        #         # Add more stages as needed
        #     },
        #     # Add more crops as needed
        # }

        nutrient_requirements_nitrogen = (
            crop_total_nitrogen_demand * self.ratio_nitrogen
        )
        nutrient_requirements_porphorus = (
            crop_total_porphorus_demand * self.ratio_porphorus
        )
        nutrient_requirements_kassium = crop_total_kassium_demand * self.ratio_kassium

        # Residual nutrients from previous applications (hypothetical)
        # 基肥的话，残差肥料都是 0 ;
        residual_n = 0  # Residual Nitrogen
        residual_p = 0  # Residual Phosphorus
        residual_k = 0  # Residual Potassium

        # Calculate required fertilizers
        # required_n = nutrient_requirements[crop_type][crop_phenology]["N"] - (soil_n + residual_n)
        # required_p = nutrient_requirements[crop_type][crop_phenology]["P"] - (soil_p + residual_p)
        # required_k = nutrient_requirements[crop_type][crop_phenology]["K"] - (soil_k + residual_k)
        required_n = nutrient_requirements_nitrogen - (soil_n + residual_n)
        required_p = nutrient_requirements_porphorus - (soil_p + residual_p)
        required_k = nutrient_requirements_kassium - (soil_k + residual_k)

        # Ensure no negative values (i.e., no need to apply more than required)
        required_n = max(required_n, 0)
        required_p = max(required_p, 0)
        required_k = max(required_k, 0)

        # Log the required fertilizer amounts
        # logging.info(f"Fertilizer required: N={required_n} kg/ha, P={required_p} kg/ha, K={required_k} kg/ha")
        logging.info(
            f"Fertilizer required: N={required_n:.2f} kg/ha, P={required_p:.2f} kg/ha, K={required_k:.2f} kg/ha"
        )

        # 如果是混合肥，根据 NPK 比例计算施肥量
        if self.fertilizer_form == "mixed":
            ratio_n, ratio_p, ratio_k = self.fertilizer_npk_ratio

            # 计算最小的混合肥用量（基于三者的比例）
            required_fertilizer_n = required_n / (ratio_n / 100)
            required_fertilizer_p = required_p / (ratio_p / 100)
            required_fertilizer_k = required_k / (ratio_k / 100)

            # 所需的混合肥总量是确保所有需求得到满足的最小量
            min_fertilizer_amount = max(
                required_fertilizer_n, required_fertilizer_p, required_fertilizer_k
            )

            # 根据 zone_indexes 计算施肥量
            # 假设 NDVI 越高，肥料需求越少，使用 (1 - NDVI) 调节施肥量
            fertilizer_values = []
            # for index, zone_ndvi in enumerate(self.zone_indexes):
            for (
                index,
                zone_ndvi,
            ) in zone_indexes.items():  # 使用 .items() 来获取键和值
                zone_fertilizer_amount = min_fertilizer_amount * (
                    1 - zone_ndvi
                )  # 根据 NDVI 调整施肥量
                zone_fertilizer_amount = float(math.ceil(zone_fertilizer_amount))
                fertilizer_values.append(zone_fertilizer_amount)
                logging.info(
                    f"Fertilizer required for zone {index + 1} (NDVI={zone_ndvi}): {zone_fertilizer_amount:.2f} kg/ha"
                )
            fertilizer_values.sort()
            return fertilizer_values

        else:
            raise ValueError("Currently only mixed fertilizer is supported.")

        # if fertilizer_form == "mixed_fertilizer":
        #     # 如果是混合肥，根据NPK比例计算所需混合肥的最小量
        #     fertilizer_npk_ratio = self.dpm_settings.get("fertilizer_npk_ratio", [15, 15, 15])
        #     ratio_n, ratio_p, ratio_k = fertilizer_npk_ratio

        #     # 计算最小的混合肥用量（基于三者的比例）
        #     required_fertilizer_n = required_n / (ratio_n / 100)
        #     required_fertilizer_p = required_p / (ratio_p / 100)
        #     required_fertilizer_k = required_k / (ratio_k / 100)

        #     # 所需的混合肥总量是确保所有需求得到满足的最小量
        #     min_fertilizer_amount = max(required_fertilizer_n, required_fertilizer_p, required_fertilizer_k)

        #     # 输出施肥量
        #     logging.info(f"Mixed fertilizer required: {min_fertilizer_amount:.2f} kg/ha")
        #     return {"Mixed Fertilizer": min_fertilizer_amount}

        # elif fertilizer_form == "single_fertilizer":
        #     # 如果是单质肥，返回每种单质肥的量
        #     logging.info(f"Single fertilizers required: N={required_n:.2f} kg/ha, P={required_p:.2f} kg/ha, K={required_k:.2f} kg/ha")
        #     return {"N": required_n, "P": required_p, "K": required_k}

        # else:
        #     raise ValueError("Unknown fertilizer form specified in dpm_settings")

    # 依据养分平衡方程，判断追肥的施肥量
    # def nutrients_balance_calculation_follow_fertilization(self):



import cv2
import numpy as np
import matplotlib.pyplot as plt
# from skimage.feature import graycomatrix, graycoprops
from osgeo import gdal
# import os
# import subprocess

# 设置matplotlib中文字体
plt.rcParams['font.sans-serif'] = ['SimHei']
plt.rcParams['axes.unicode_minus'] = False

def process_ngrdi(image_path):
    """处理叶片图像并提取掩码
    
    Args:
        image_path: 图像路径
        show_visualization: 是否显示可视化结果
    
    Returns:
        tuple: (处理后的图像, 叶片掩码, 原始图像)
    """

    # 切割为小图片，逐个处理。因为不确定工作目录的情况，所以暂时注释掉
    '''
    # 把大的tif切割成多个4000*3500的小块，这样内存足够处理
    tmp_dir = os.path.dirname(image_path) + '/tmp/'
    if not os.path.exists(tmp_dir):
        os.mkdir(tmp_dir)
    command = f"gdal_retile.py -ps 4000 3500 -targetDir {tmp_dir} {image_path}"

    try:
        subprocess.check_call(command, shell=True)
    except subprocess.CalledProcessError as e:
        print(f"Subprocess failed with code {e.returncode}")
    
    sub_images = os.listdir(tmp_dir)
    for sub_image in sub_images:
        print (sub_image)
    '''

    # 读取图像
    img = cv2.imread(image_path)
    ## 读取图像
    #nparr = np.frombuffer(image_bytes, np.uint8)
    ## Decode image
    #img = cv2.imdecode(nparr, cv2.IMREAD_COLOR)

    height, width = img.shape[:2]

    dataset1 = gdal.Open(image_path)
    projection = dataset1.GetProjection()
    geotransform = dataset1.GetGeoTransform()

    return img, projection, geotransform
    

def get_display_region(mask):
    """获取有效显示区域的边界框"""
    rows = np.any(mask > 0, axis=1)
    cols = np.any(mask > 0, axis=0)
    ymin, ymax = np.where(rows)[0][[0, -1]]
    xmin, xmax = np.where(cols)[0][[0, -1]]
    
    # 添加边距
    padding = 10
    ymin = max(0, ymin - padding)
    ymax = min(mask.shape[0], ymax + padding)
    xmin = max(0, xmin - padding)
    xmax = min(mask.shape[1], xmax + padding)
    
    return ymin, ymax, xmin, xmax

def calculate_ngrdi(image):
    """计算NDVI指数"""
    b, g, r = cv2.split(image)
    #mask_bool = mask > 0
    
    # 创建带NaN的数组
    pseudo_ndvi = np.full_like(g, np.nan, dtype=float)
    
    # 只在mask区域内计算
    denominator = (g.astype(float) + r.astype(float))
    #valid = (denominator != 0) & mask_bool
    #pseudo_ndvi[valid] = (g[valid].astype(float) - r[valid].astype(float)) / denominator[valid]
    pseudo_ndvi = (g.astype(float) - r.astype(float)) / denominator
    
    return pseudo_ndvi

#def analyze_leaf(image_path, show_visualization=True):
def gen_ngrdi(in_tif, out_tif):
    """综合分析叶片特征"""
    # 处理图像
    #result, mask, original = process_leaf_image(image_path, show_visualization=False)
    original, projection, geotransform = process_ngrdi(in_tif)
    
    # 计算各项指标
    ngrdi = calculate_ngrdi(original)
    result = cv2.imwrite(out_tif, ngrdi)
    dataset = gdal.Open(out_tif, gdal.GA_Update)
    dataset.SetGeoTransform( geotransform )
    dataset.SetProjection( projection )
    
    
    return True

if __name__ == "__main__":
    # image_path = '/Users/baoyonghui/Downloads/cropmirror-utils-test/rgb_orthophoto_2025_03_27_110432.tif'
    image_path = 'D://cropmirror//cropmirror-utils//src//rgb//rgb_orthophoto_2025_01_09_222535.tif'
    try:
        # results = gen_ngrdi(image_path, '/Users/baoyonghui/Downloads/cropmirror-utils-test/result.tif')
        results = gen_ngrdi(image_path, 'D://cropmirror//cropmirror-utils//src//rgb//rgb_orthophoto_2025_01_09_222535_result.tif')
        print (results)
    except Exception as e:
        print(f"处理过程中出现错误: {str(e)}") 


# 生成播种所需要的 values;
# v1: 根据地形的变化进行变量播种；
#

class Seeding():
    def __init__(
        self,
        path_prefix,
        uuid,
        name,
        geometry,
        starttime="",  # 2024-
    ):
        pass


import logging
import rasterio
from scipy.ndimage import gaussian_filter
import numpy as np
import os

from osgeo import gdal, ogr, osr

def reclassify(geotiff_filepath, reclassify_filepath, level_num: int = 5):
    """
    使用自然间断点方法，将裁剪后的geotiff进行分级, 生成reclassify file
    参数:
    geotiff_filepath: 原始geotiff 文件
    reclassify_filepath (str): 分级后的geotiff保存文件名
    num (int): 分级数目
    """

    # 读取裁剪后的geotiff文件
    with rasterio.open(geotiff_filepath) as src:
        ndvi = src.read(1)
        profile = src.profile

    gaussian_ndvi = gaussian_filter(
        ndvi, sigma=3
    )  # Apply Gaussian filter for density estimation

    logging.info(np.nanmin(gaussian_ndvi))
    logging.info(np.nanmax(gaussian_ndvi))
    # 计算自然间断点

    # todo
    breaks = np.linspace(
        np.nanmin(gaussian_ndvi), np.nanmax(gaussian_ndvi), int(level_num)
    )

    # 重新分类数据
    reclassified_data = np.digitize(gaussian_ndvi, breaks)

    # 更新profile信息
    profile.update(count=1)

    # 保存重新分类后的geotiff文件
    with rasterio.open(reclassify_filepath, "w", **profile) as dst:
        dst.write(reclassified_data, 1)

    logging.info(
        f"已成功将{geotiff_filepath}文件重新分类为 {level_num} 级: {reclassify_filepath}"
    )

def raster2vector(reclassfy_filepath,shp_filepath):

    # path_split = os.path.split(raster_path)
    # file_split = path_split[-1].split('.')
    # vector_path = path_split[0] + '/' + file_split[0] + '.shp'
    """栅格转矢量"""
    # 打开栅格数据
    raster = gdal.Open(reclassfy_filepath)
    band = raster.GetRasterBand(1)

    # 获取投影信息
    proj = raster.GetProjection()
    # geotransform = raster.GetGeoTransform()

    # 创建空间参考
    spatial_ref = osr.SpatialReference()
    spatial_ref.ImportFromWkt(proj)

    # 创建Shapefile
    driver = ogr.GetDriverByName("ESRI Shapefile")
    shp = driver.CreateDataSource(shp_filepath)
    # 使用与 dpm_shp_filepath 同名的图层名称
    layer_name = os.path.splitext(os.path.basename(shp_filepath))[0]
    # 创建图层，并设置空间参考
    layer = shp.CreateLayer(layer_name, srs=spatial_ref, geom_type=ogr.wkbPolygon)

    # 添加字段
    field_name = ogr.FieldDefn("value", ogr.OFTInteger)
    layer.CreateField(field_name)

    # 矢量化
    band.SetNoDataValue(0)
    mask_band = band.GetMaskBand()
    gdal.Polygonize(
        band, maskBand=mask_band, outLayer=layer, iPixValField=0, callback=None
    )

    # # 创建空间参考（设置 CRS）
    # spatial_ref = osr.SpatialReference()
    # spatial_ref.ImportFromWkt(proj)
    # layer_definition = layer.GetLayerDefn()
    # layer.SetSpatialRef(spatial_ref)

    # 确认 CRS 信息是否存在
    if layer.GetSpatialRef() is not None:
        print("CRS information has been successfully set.")
        print(layer.GetSpatialRef().ExportToWkt())
    else:
        print("No CRS information found in the layer.")

    # # 添加空间参考
    # spatial_ref = osr.SpatialReference(wkt=proj)
    # layer.SetSpatialFilter(spatial_ref)

    # # 添加几何变换参数
    # layer.SetGeoTransform(geotransform)

    # 关闭数据集
    raster = None
    shp = None



import logging
from ..generatevalues.fertilization import Fertilization
from .files import NdviFiles
from ...utils.geotiff_toolkit.reclassify import reclassify, raster2vector
from ...utils.shp_toolkit.toolkit import clip_shp_by_polygon, ShpToolkit
from .common import valued_dpm_shp_postpro


class FertilizationPrescription(Fertilization):
    def __init__(self, ndvi_tif, geometry: dict, num=5, workspace=".", **kwargs):
        Fertilization.__init__(self, **kwargs)
        self._ndvi_tif = ndvi_tif
        self._num = num
        self._geometry = geometry
        self._workspace = workspace
        self._files = NdviFiles(self._workspace)
        pass

    def run(self):
        reclassify(self._ndvi_tif, self._files._reclassify_file, level_num=self._num)
        raster2vector(self._files._reclassify_file, self._files._dpm_shp_file)
        # 首先需要根据处方图的类型，判断调用哪种处方图的计算函数，是底肥，追肥；
        # 追肥的话，这里需要添加养分平衡方程，而且需要先对接口传过来的数据进行判断，
        # 如果接口传过来的数据中values值不是空，那么就调用这个函数，
        # 否则就调用养分平衡方程进行计算，需要知道土壤里面有多少氮、磷、钾（碱解氮、速效磷、速效钾需要进行换算），
        # 目前这种 crop_type （作物类型）处在这种生育期下应该需要多少氮、磷、钾，结合此种作物当前生育期下的吸收率；
        # 还需要考虑上次施肥剩下的氮，是速效肥还是缓释肥，考虑气象条件，目前还剩下多少氮、磷、钾。最后计算出来应该追多少肥。底肥的话，只考虑土壤及作物品种就行。

        # 氮的需求量应该还受NDVI的调控；
        # self.agri_operation_input_values = self.nutrients_balance_calculation_base_fertilization()

        values = self.nutrients_balance_calculation_base_fertilization(
            self._ndvi_tif, self._files._dpm_shp_file
        )

        clip_shp_by_polygon(
            self._files._dpm_shp_file, self._geometry, self._files._clipped_shp_file
        )
        shtool = ShpToolkit(self._files._clipped_shp_file)
        shtool.insert_attributes(values=values)
        shtool.save(self._files._valued_shp_file)
        shtool.save(self._files._geojson_file, driver="GeoJSON")

        variable_total, uniform_total, saving_rate = valued_dpm_shp_postpro(
            self._files._valued_shp_file, self.fertilizer_average_value_manual
        )  # 依据生成的处方图，计算处方作业总施肥量、变量作业总施肥量、节肥率；

        logging.info(
            f"变量施肥总量: {variable_total} kg\n"
            f"匀量施肥总量: {uniform_total} kg\n"
            f"节肥率: {saving_rate}%"
        )



from ndvi.preprocess.ndvi import Ndvi


class VigorMonitoring(Ndvi):
    def __init__(self, ndvi_tif: str):
        """初始化 解析ndvi_tif 文件

        Args:
            ndvi_tif (str): ndvi tif 文件路径
        """
        super().__init__(ndvi_tif=ndvi_tif)
        
        pass

    def vigor_monitoring(self,boundary_file: str, savepath: str):
        """长势分析

        Args:
            boundary_file (str): 边界文件,根据后缀shp/geojson 自动识别
            savepath (str): 结果存储路径, 文件名vigor_20250322_1407,格式包含shp/geojson
        """
        print("vigor monitoring")
        pass
    
    def biomass_estimation(self,boundary_file: str, savepath: str):
        """生物量估算

        Args:
            boundary_file (str): 边界文件,根据后缀shp/geojson 自动识别
            savepath (str): 结果存储路径, 文件名vigor_20250322_1407,格式包含shp/geojson
        """
        print("biomass estimation")
        pass



import json
import os
import geopandas as gpd
from osgeo import gdal, ogr, osr
import tempfile
import shutil
import time
import matplotlib.pyplot as plt
import logging
import datetime
from shapely.geometry import Point
from geopy.geocoders import Nominatim
import matplotlib.font_manager as fm
import numpy as np
import matplotlib.colors as colors

# 配置日志
logging.basicConfig(
    level=logging.INFO,
    format="%(asctime)s %(levelname)s %(pathname)s %(lineno)d  %(message)s",
    datefmt="%Y-%m-%dT%H:%M:%S",
)


here = os.path.dirname(__file__)
# geojsonfile = os.path.join(here, "prescription.geojson")
geojsonfile = os.path.join(here, "test.geojson")


# Convert GeoJSON to SHP
def convert_shp(
    data, output_dir, suffix=datetime.datetime.now().strftime("%Y%m%d%H%M%S")
):
    filename = "%s.shp" % (suffix)
    savepath = os.path.join(output_dir, filename)
    if not os.path.exists(output_dir):
        os.makedirs(output_dir)

    gdf = gpd.GeoDataFrame.from_features(data["features"])

    # 检查 GeoDataFrame 是否有 CRS 信息
    if gdf.crs is None:
        logging.info("No CRS information found, setting CRS to WGS84 (EPSG:4326).")
        gdf.set_crs(epsg=4326, inplace=True)

    # 确保 CRS 已经被正确设置
    if gdf.crs is None or gdf.crs != "EPSG:4326":
        raise ValueError("Failed to set CRS to WGS84 (EPSG:4326).")

    gdf.to_file(savepath, driver="ESRI Shapefile")

    gdf_check = gpd.read_file(savepath)
    logging.info(f"Saved Shapefile CRS: {gdf_check.crs}")


# Convert GeoJSON to TIFF, 大疆植保机；tif, tfw; tif:; .clr;
def convert_tif(
    data, output_dir, dpm_res=4, suffix=datetime.datetime.now().strftime("%Y%m%d%H%M%S")
):
    filename = "%s.tif" % (suffix)
    savepath = os.path.join(output_dir, filename)
    if not os.path.exists(output_dir):
        os.makedirs(output_dir)
    gdf = gpd.GeoDataFrame.from_features(data["features"])

    # 检查GeoDataFrame是否有CRS信息
    # if gdf.crs is None:
    #     logging.info("No CRS information found, setting CRS to WGS84 (EPSG:4326).")
    #     gdf.set_crs(epsg=4326, inplace=True)

    if "crs" not in data or data["crs"] is None or gdf.crs is None:
        logging.info("No CRS information found, setting CRS to WGS84 (EPSG:4326).")
        gdf.set_crs(epsg=4326, inplace=True)

    gdf.to_crs(epsg=4326, inplace=True)  # 确保CRS正确

    if "value" not in gdf.columns:
        raise ValueError(
            "The GeoDataFrame does not contain a 'value' column, which is required for rasterization."
        )

    # gdf['value'] = 1  # 设置用于栅格化的值，level, color;
    xmin, ymin, xmax, ymax = gdf.total_bounds
    # 计算中心纬度
    center_lat = (ymin + ymax) / 2

    # 计算栅格分辨率，目标为 4 米
    row_res = dpm_res / 111320  # 4 米转换为纬度单位
    col_res = dpm_res / (111320 * np.cos(np.radians(center_lat)))  # 4 米转换为经度单位

    cols = int((xmax - xmin) / col_res)
    rows = int((ymax - ymin) / row_res)

    # 创建临时文件夹
    temp_dir = tempfile.mkdtemp()
    try:
        # 保存矢量数据为临时Shapefile
        shp_path = os.path.join(temp_dir, "temp_shp.shp")
        gdf.to_file(shp_path, driver="ESRI Shapefile")

        # 确认文件是否成功创建
        if not os.path.exists(shp_path):
            raise FileNotFoundError(f"Temporary shapefile not found: {shp_path}")

        # 创建栅格数据集
        driver = gdal.GetDriverByName("GTiff")
        out_raster = driver.Create(savepath, cols, rows, 1, gdal.GDT_Float32)
        out_raster.SetGeoTransform((xmin, col_res, 0, ymax, 0, -row_res))

        # 设置投影信息为 WGS84 (EPSG:4326)
        srs = osr.SpatialReference()
        srs.ImportFromEPSG(4326)
        out_raster.SetProjection(srs.ExportToWkt())

        outband = out_raster.GetRasterBand(1)

        # 使用 NaN 作为无数据值
        outband.SetNoDataValue(np.nan)

        # 查看文件夹中的文件列表，确保 Shapefile 相关文件都存在
        logging.info(f"Temporary directory contents: {os.listdir(temp_dir)}")

        # 使用 OGR 打开 Shapefile
        shp_ds = ogr.Open(shp_path)
        if shp_ds is None:
            raise RuntimeError(f"Failed to open shapefile: {shp_path}")

        # 获取矢量图层
        layer = shp_ds.GetLayer()

        # 使用临时矢量文件进行栅格化
        gdal.RasterizeLayer(out_raster, [1], layer, options=["ATTRIBUTE=value"])

        outband.FlushCache()

        # 显式释放数据源
        shp_ds = None

        # 保存颜色映射为 .clr 文件
        clr_path = os.path.splitext(savepath)[0] + ".clr"
        unique_values = gdf["value"].unique()
        num_colors = len(unique_values)

        # 使用 matplotlib 中的 ListedColormap 来生成有序的颜色映射
        cmap = plt.get_cmap(
            "viridis", num_colors
        )  # 选择一个有序的颜色映射（例如 viridis）

        # 将颜色映射保存到 .clr 文件
        with open(clr_path, "w") as f:
            for i, value in enumerate(unique_values):
                # # 为每个唯一值生成随机颜色
                # rgb = (random.randint(0, 255), random.randint(0, 255), random.randint(0, 255))
                # f.write(f"{value} {rgb[0]} {rgb[1]} {rgb[2]}\n")
                # 获取颜色映射中的 RGB 值（在 0-1 范围内），然后转换到 0-255 范围
                r, g, b, _ = cmap(i / (num_colors - 1))  # RGBA 其中 A 忽略
                r, g, b = int(r * 255), int(g * 255), int(b * 255)
                f.write(f"{value} {r} {g} {b}\n")

    finally:
        # 在删除临时文件夹之前等待一会
        time.sleep(1)
        # 删除临时文件夹
        shutil.rmtree(temp_dir)


# Convert GeoJSON to ISOXML version 4
from xml.etree.ElementTree import Element, SubElement, ElementTree
import struct
import zipfile
import os


def convert_isoxml_v4(data, output_dir, suffix=datetime.datetime.now().strftime("%Y%m%d%H%M%S")):
    # 创建目标文件夹（如果不存在）
    if not os.path.exists(output_dir):
        os.makedirs(output_dir)

    # 创建 XML 文件，文件名为 TASKDATA.XML
    xml_filename = "TASKDATA.XML"
    xml_path = os.path.join(output_dir, xml_filename)
    # 创建对应的 .bin 文件
    bin_filename = "TASKDATA.BIN"
    bin_path = os.path.join(output_dir, bin_filename)

    # 创建根节点 ISO11783_TaskData
    root = Element(
        "ISO11783_TaskData",
        {
            "VersionMajor": "2",
            "VersionMinor": "0",
            "ManagementSoftwareManufacturer": "CropMirror",
            "ManagementSoftwareVersion": "1.0",
            "TaskControllerVersion": "1.0",
            "DataTransferOrigin": "1",
        },
    )

    # 创建 TSK 节点 (Task)
    tsk = SubElement(
        root,
        "TSK",
        {
            "A": "TSK1",
            "B": "20240402104938_valued_dpm",  # 可以自定义任务名称
            "G": "1",
            "H": "1",
        },
    )

    # 创建 DLT 节点
    dlt = SubElement(tsk, "DLT", {"A": "DFFF", "B": "31", "D": "1000"})

    # 创建 GRD 节点 (Grid)
    grd = SubElement(
        tsk,
        "GRD",
        {
            "A": "37.06903967",
            "B": "115.26926947",
            "C": "0.00008730",
            "D": "0.00011239",
            "E": "13",
            "F": "22",
            "G": "GRD00000",
            "I": "2",
            "J": "2",
        },
    )

    # 添加 TreatmentZone 节点
    for i, feature in enumerate(data["features"]):
        tzn = SubElement(tsk, "TZN", {"A": str(i), "B": f"Zone_{i}", "C": "0"})

        # 添加 PDV 节点 (施肥量或其他参数)
        properties = feature.get("properties", {})
        value = properties.get("value", 0)  # 如果 GeoJSON 中存在 'value' 属性，则读取它
        pdv = SubElement(
            tzn,
            "PDV",
            {"A": "0006", "B": str(int(value))},  # 使用从 GeoJSON 中获取的 'value' 值
        )

    # 保存 XML 文件
    tree = ElementTree(root)
    with open(xml_path, "wb") as f:
        tree.write(f, encoding="utf-8", xml_declaration=True)

    with open(bin_path, "wb") as bin_file:
        for feature in data["features"]:
            properties = feature.get("properties", {})
            value = properties.get(
                "value", 0
            )  # 假设 value 是需要存储的值，如果不存在则为 0

            # 假设将 value 写入 .bin 文件，以 float 格式存储
            bin_file.write(struct.pack("<f", float(value)))

    # 创建 ZIP 压缩包
    # zip_path = os.path.join(output_dir, "ISOXML_Output.zip")
    # with zipfile.ZipFile(zip_path, 'w') as zipf:
    #     zipf.write(xml_path, xml_filename)
    #     zipf.write(bin_path, bin_filename)

    # logging.info(f"ISOXML and BIN files created and zipped: {zip_path}")


# Convert GeoJSON to ISOXML version 5
def convert_isoxml_v5(data, output_dir):
    filename = "%s.xml5" % (datetime.datetime.now().strftime("%Y%m%d%H%M%S"))
    savepath = os.path.join(output_dir, filename)
    if not os.path.exists(output_dir):
        os.makedirs(output_dir)

    with open(savepath, "w") as f:
        f.write('<ISO11783_TaskData Version="5">\n')
        # Convert data['features'] to ISOXML format...
        f.write("</ISO11783_TaskData>")


def show_tif(tif_path):
    # 打开 TIFF 文件
    dataset = gdal.Open(tif_path)
    if dataset is None:
        raise FileNotFoundError(f"Failed to open TIFF file: {tif_path}")

    # 获取栅格数据
    band = dataset.GetRasterBand(1)
    array = band.ReadAsArray()

    # 自动生成 clr 文件路径
    clr_path = os.path.splitext(tif_path)[0] + ".clr"

    # 读取颜色映射文件，如果提供了 .clr 文件
    if os.path.exists(clr_path):
        # 创建颜色字典来保存栅格值与对应的 RGB 颜色
        value_to_color = {}
        with open(clr_path, "r") as clr_file:
            for line in clr_file:
                if line.strip():
                    parts = line.split()
                    value = float(parts[0])
                    rgb = tuple(map(int, parts[1:4]))
                    value_to_color[value] = rgb

        # 创建一个与栅格大小相同的 RGB 图像数组
        rgb_image = np.zeros((array.shape[0], array.shape[1], 3), dtype=np.uint8)

        # 将每个栅格值映射到对应的 RGB 颜色
        for value, color in value_to_color.items():
            mask = array == value
            rgb_image[mask] = color

        # 显示 RGB 图像
        plt.imshow(rgb_image)
    else:
        # 使用默认的 viridis 颜色映射进行显示
        plt.imshow(array, cmap="viridis")

    plt.title("TIFF Visualization")
    plt.colorbar()
    plt.show()
    logging.info("TIFF visualization completed.")


def get_administrative_area(lat, lon):
    geolocator = Nominatim(user_agent="geo_shapefile_viewer")
    location = geolocator.reverse((lat, lon), language="zh")
    if location and location.raw.get("address"):
        address = location.raw["address"]
        admin_area = f"{address.get('state', '')} {address.get('county', '')} {address.get('town', '')}"
        return admin_area.strip()
    else:
        return "Unknown Location"


def show_shp(shp_path):
    # 读取 Shapefile
    gdf = gpd.read_file(shp_path)

    # 获取 Shapefile 的几何中心点
    centroid = gdf.geometry.centroid.unary_union.centroid
    if isinstance(centroid, Point):
        lon, lat = centroid.x, centroid.y
        # 使用逆地理编码查找行政区名称
        admin_area = get_administrative_area(lat, lon)
    else:
        admin_area = "Unknown Location"

    # 检查是否存在 'value' 列
    if "value" not in gdf.columns:
        logging.warning("'value' column not found in Shapefile. Using default colors.")
        gdf.plot()
    else:
        # 显示 Shapefile，根据 'value' 列设置不同的颜色
        gdf.plot(column="value", legend=True, cmap="viridis")

    # 设置中文字体
    plt.rcParams["font.sans-serif"] = ["SimHei"]  # 用黑体显示中文
    plt.rcParams["axes.unicode_minus"] = False  # 正常显示负号

    plt.title(f"Shapefile Visualization - {admin_area}")
    plt.xlabel("Longitude")
    plt.ylabel("Latitude")
    plt.show()
    logging.info("Shapefile visualization completed.")


if __name__ == "__main__":

    current_timestamp = datetime.datetime.now().strftime("%Y%m%d%H%M%S")
    shp_path = os.path.join(here, f"{current_timestamp}_shp")
    tif_path = os.path.join(here, f"{current_timestamp}_tif")
    # xml_v4_path = os.path.join(here, f"{current_timestamp}_tmp_v4.xml")
    xml_v4_path = os.path.join(here, f"{current_timestamp}_xml_v4")
    # xml_v5_path = os.path.join(here, f"{current_timestamp}_tmp_v5.xml")
    xml_v5_path = os.path.join(here, f"{current_timestamp}_xml_v5")

    # Read GeoJSON file
    with open(geojsonfile) as f:
        data = json.load(f)  # Load file content into data

    # Check if CRS information is available
    if "crs" in data:
        logging.info("CRS Type: %s", data["crs"]["type"])
        logging.info("CRS Properties: %s", data["crs"]["properties"])
    else:
        logging.info("No CRS information available in this GeoJSON.")

    logging.info(data["type"])

    # Convert to SHP
    convert_shp(data, shp_path)

    # 添加 shp 可视化的代码

    # 显示 Shapefile
    # show_shp(shp_path)

    # # Convert to TIFF
    convert_tif(data, tif_path, dpm_res=4)

    # # 添加显示 tmp.tif 的代码；
    # show_tif(tif_path)

    # # Convert to ISOXML v4
    convert_isoxml_v4(
        data, xml_v4_path
    )  # 这传入的应该是个文件夹路径，然后在这个文件夹下面生成 TASKDATA.xml 和 TASKDATA.bin

    # # Convert to ISOXML v5
    # convert_isoxml_v5(data, xml_v5_path)



import geopandas as gpd
from math import sqrt
from shapely import wkt
from shapely.geometry import mapping
import geojson
import json
from shapely.geometry import Point, Polygon, MultiPolygon, GeometryCollection
from shapely.geometry import shape
def generate_polygon(center_lat, center_lon, distance=805, cap_style="round"):
    gs = gpd.GeoSeries(wkt.loads(f"POINT ({center_lon} {center_lat})"))
    gdf = gpd.GeoDataFrame(geometry=gs)
    gdf.crs = "EPSG:4326"
    gdf = gdf.to_crs("EPSG:3857")
    res = gdf.buffer(
        distance=distance,
        cap_style=cap_style,
    )
    geojson_string = geojson.dumps(
        mapping(wkt.loads(res.to_crs("EPSG:4326").iloc[0].wkt))
    )

    
    geojson_dict = json.loads(geojson_string)
    
    polygon = shape(geojson_dict)
    gdf = gpd.GeoDataFrame(index=[0], crs='epsg:4326', geometry=[polygon])
    area = gdf.to_crs(32649).area.iloc[0] /1000000
    print(area)
    return geojson_dict

from shapely.geometry import box
from geopy.distance import geodesic
import json

def generate_rectangle(center_lat, center_lon, width_m, height_m):
    # 计算矩形宽度和高度对应的纬度和经度变化量
    half_width_delta = geodesic(meters=width_m / 2).destination((center_lat, center_lon), bearing=90).longitude - center_lon
    half_height_delta = geodesic(meters=height_m / 2).destination((center_lat, center_lon), bearing=0).latitude - center_lat

    # 计算矩形的边界坐标
    minx = center_lon - half_width_delta
    maxx = center_lon + half_width_delta
    miny = center_lat - half_height_delta
    maxy = center_lat + half_height_delta

    # 使用 box 函数生成矩形
    rectangle = box(minx, miny, maxx, maxy)
    
    # 将 Polygon 对象转换为 GeoJSON 格式
    rectangle_geojson = {
        "type": "Polygon",
        "coordinates": [list(rectangle.exterior.coords)]
    }
    
    return rectangle_geojson


if __name__ == "__main__":
    generate_polygon(center_lat=47.287796,center_lon=132.690268,distance=805)



import logging
import math

from .common import get_zone_indexes
# 创建 class Irrigation
# long.liu89@hotmail.com
# 2025-2-2 10:48:10

# 依据水量平衡方程，判断灌溉的需水量
def water_balance_calculation_base_irrigation( ndvi_geotiff_file, valued_shp_file, 
                                                precipitation, evapotranspiration, 
                                                runoff, deep_percolation, 
                                                initial_soil_moisture, target_soil_moisture):
    """
    计算基于水量平衡方程的基本灌溉需求。
    
    参数:
    precipitation (float): 降水量 (毫米)
    evapotranspiration (float): 蒸发蒸腾量 (毫米)
    runoff (float): 地表径流量 (毫米)
    deep_percolation (float): 深层渗漏量 (毫米)
    initial_soil_moisture (float): 初始土壤含水量 (毫米)
    target_soil_moisture = 300  # 假设目标土壤含水量为300毫米
    
    返回:
    float: 推荐灌溉量 (立方米/亩) 1升 = 1000立方米
    """
    # 计算土壤储水量的变化
    delta_s = precipitation - evapotranspiration - runoff - deep_percolation
    
    # 根据实际需要调整土壤含水量到目标值所需的灌溉量（这里简化处理）
    
    irrigation_needed = max(0, target_soil_moisture - (initial_soil_moisture + delta_s))
    
    # 假设每亩面积为667平方米，转换为立方米水
    # irrigation_volume_liters_per_mu = irrigation_needed * 667
    irrigation_volume_m3_per_mu = irrigation_needed * 667

    zone_indexes = get_zone_indexes(ndvi_geotiff_file,valued_shp_file)
    #
    irrigation_volume_m3_values = []
    # for index, zone_ndvi in enumerate(self.zone_indexes):
    for (
        index,
        zone_ndvi,
    ) in zone_indexes.items():  # 使用 .items() 来获取键和值
        zone_irrigation_volume = irrigation_volume_m3_per_mu * (zone_ndvi)  
        # 根据 NDVI 调整施药量
        # if zone_ndvi < 0.3:
        #     zone_pesticide_dose *= 1.5
        # elif zone_ndvi >= 0.3 and zone_ndvi < 0.6:
        #     zone_pesticide_dose *= 1.0
        # elif zone_ndvi >= 0.6:
        #     zone_pesticide_dose *= 0.5

        zone_irrigation_volume = float(math.ceil(zone_irrigation_volume*100.0)/100.0)
        irrigation_volume_m3_values.append(zone_irrigation_volume)
        logging.info(
            f"Irrigation required for zone {index + 1} (NDVI={zone_ndvi}): {zone_irrigation_volume:.2f} L/mu"
        )

    irrigation_volume_m3_values.sort()

    return irrigation_volume_m3_values, irrigation_volume_m3_per_mu
    
    # return irrigation_volume_m3_per_mu




import logging
import geopandas as gpd
import json
import os

def shp2geojson(input_shp_file, output_geojson_file):
    # 检查输入文件是否存在
    if not os.path.exists(input_shp_file):
        logging.error("输入Shapefile文件不存在，请检查路径：%s", input_shp_file)
        raise

    try:
        # 读取shp文件
        gdf = gpd.read_file(input_shp_file)
    except Exception as e:
        logging.error("读取Shapefile文件失败：%s", e)
        raise

    # 显示shapefile当前的坐标系
    logging.info("当前的坐标系 (CRS)：%s", gdf.crs)

    # 将坐标系转换为WGS84
    gdf = gdf.to_crs(epsg=4326)

    gdf.to_file(output_geojson_file, driver="GeoJSON")

    logging.info("GeoJSON文件已保存：%s", output_geojson_file)

def check_shp_crs(shp_file):
    if not os.path.exists(shp_file):
        logging.error("输入Shapefile文件不存在，请检查路径：%s", shp_file)
        return None

    gdf = gpd.read_file(shp_file)
    crs = gdf.crs  # 获取CRS信息

    if crs:
        print(f"Shapefile CRS: {crs}")
    else:
        print("Shapefile不包含CRS信息。")
    return crs

def check_geojson_crs(geojson_file):
    # 检查GeoJSON文件是否存在
    if not os.path.exists(geojson_file):
        logging.error("GeoJSON文件不存在，请检查路径：%s", geojson_file)
        return

    # 读取 GeoJSON 文件
    with open(geojson_file) as f:
        data = json.load(f)

    # 检查是否有 CRS 信息
    if 'crs' in data:
        print("CRS Type:", data['crs']['type'])
        print("CRS Properties:", data['crs']['properties'])
    else:
        print("GeoJSON 文件中没有 CRS 信息。")

if __name__ == "__main__":
    # 配置日志
    logging.basicConfig(level=logging.INFO, format="%(asctime)s %(levelname)s %(pathname)s %(lineno)d  %(message)s", datefmt="%Y-%m-%dT%H:%M:%S")

    # 输入文件路径
    here = os.path.dirname(__file__)
    input_shp_file = os.path.join(here, "valued_dpm_shp_file.shp")
    output_geojson_file = os.path.join(here, "test.geojson")

    # 检查Shapefile的CRS
    check_shp_crs(input_shp_file)

    # 将Shapefile转换为GeoJSON
    shp2geojson(input_shp_file, output_geojson_file)

    # 检查GeoJSON文件的CRS
    check_geojson_crs(output_geojson_file)
